<?php

include_once './constants.php';
require_once __DIR__ . '/protocolHeaderPartParser.php';

/**
 * compiles the given main file (if a compile error occurs then an answer is sent)
 * @param $conn PDO
 * @param $traceString string
 * @param $sourceCode string the user program source code as one string
 * @param string $mainFileNameWithExtension the main file to compile (with extension)
 * @param $fullWorkingDirPath string the working dir with the source files
 * @param int|string $compileTimeout the compile timeout in MS  (can be string from db or int from default values)
 * @param string $compileCmd the compile command to execute
 * @param $sourceFileExtensions array the list with the file extensions from the source files for the p language
 * @param $uniqueSessionId string the unique session id for the test runner to handle sandbox file system management
 * @param $showTestRunnerDebugOutput true: show internal debug output from the test runner (included in the test response)
 * @param $compilerOptions string the options for the compiler
 * @param $characterLimitProtocol int the max characters for the test result protocol
 * @return array Format: return_var: process exit value, output the output of the process
 * Format: return_var: process exit value, output the output of the process
 * @internal param array $allFiles all files to copy in teh dir to use
 * Format: array[int] => {fileName: string, fileContent: string}
 */
function do_compile($conn, $traceString, $sourceCode, $mainFileNameWithExtension, $fullWorkingDirPath,
                    $compileTimeout,
                    $compileCmd, $sourceFileExtensions,
                    $uniqueSessionId,
                    $showTestRunnerDebugOutput,
                    $compilerOptions,
                    $characterLimitProtocol
)
{

    global $isDebug;
    global $config;
    global $s_output;
    global $s_return_val;
    global $s_command_compile;
    global $s_user_program_exit_code;
    global $s_runnerVersion;
    global $s_timeForCompiling;
    global $s_timeForUserProgram;
    global $s_timeoutInMsUsed;
    global $s_compileTimeoutInMsUsed;
    global $s_output_without_header_part;
    global $s_characterLimitExceeded;

    $command_to_execute_string = $s_command_compile;

    $output = [];
    $return_var = -1;

    $longCmd = $config['runner']
        . ' ' . escapeshellarg($command_to_execute_string)                          # arg[0] the test method
        . ' ' . escapeshellarg($fullWorkingDirPath)                                 # arg[1] dir path
        . ' ' . escapeshellarg($mainFileNameWithExtension)                          # arg[2] file path
        . ' ' . escapeshellarg($compileTimeout)                                     # arg[3] compile timeout in ms
        . ' ' . escapeshellarg($compileCmd)                                         # arg[4] command to execute
        . ' ' . escapeshellarg(implode(',', $sourceFileExtensions))           # arg[5] the source file extensions
        . ' ' . escapeshellarg($uniqueSessionId)                                    # arg[6] a unique session id
        . ' ' . escapeshellarg($showTestRunnerDebugOutput === TRUE ? 1 : 0)    # arg[7] showTestRunnerDebugOutput
        . ' ' . escapeshellarg($compilerOptions)                                    # arg[8] options for the compiler
        . ' ' . escapeshellarg($characterLimitProtocol)                             # arg[9] character limit for the test protocol, output cut after the limit
    ;

    require_once './transactionHelper.php';
    addTraceLogToDb($conn, $config, $traceString, $longCmd, $sourceCode);

    $env = $config['environmentVarsParsed'];
//    exec($longCmd, $output, $return_var);

    /* could also be used...proc_open only works if bypass_shell is true??     */
    $descriptorspec = array(
        0 => array('pipe', 'r'), # stdin is a pipe that the child will read from
        1 => array('pipe', 'w'),  # stdout is a pipe that the child will write to
        2 => array("pipe", "w")
    );

    addToDebugTimings("starting runner");

    $process = proc_open($longCmd, $descriptorspec, $pipes, $fullWorkingDirPath, $env, array('bypass_shell' => TRUE));

    if (is_resource($process)) {

        $output = stream_get_contents($pipes[1]);
        fclose($pipes[1]);

        $errorOutput = stream_get_contents($pipes[2]);
        fclose($pipes[2]);

        $return_var = proc_close($process);
    }

    if ($isDebug && (isset($errorOutput) && trim($errorOutput) !== '')) {
        debug("error during execution of the (compile) test runner: " . $errorOutput);
    }

    addToDebugTimings("finished runner");

    $headerPart = parseHeaderPart($output);

    # in case the test runner didn't even run we may need to cast (vals from db)

    if (gettype($compileTimeout) === 'string') {
        $compileTimeout = intval($compileTimeout, 10);
    }

    return array(
        $s_return_val => $return_var,
        $s_output => $headerPart[$s_output_without_header_part],
        $s_user_program_exit_code => $headerPart[$s_user_program_exit_code],
        $s_runnerVersion => $headerPart[$s_runnerVersion],
        $s_timeForCompiling => $headerPart[$s_timeForCompiling],
        $s_timeForUserProgram => $headerPart[$s_timeForUserProgram],
        $s_timeoutInMsUsed => $headerPart[$s_timeoutInMsUsed],
        $s_compileTimeoutInMsUsed => $compileTimeout,
        $s_characterLimitExceeded => $headerPart[$s_characterLimitExceeded],
    );
}
